package tech.krave.twilio_chat_plugin

import com.twilio.chat.*
import com.twilio.chat.ChatClientListener as CCListener
import com.twilio.chat.StatusListener


class ChatClientListener : BaseListener(), CCListener{

    override fun onChannelDeleted(channel: Channel?) {
        TwilioChatPlugin.debug("""ChatClientListener.onChannelDeleted => channelSID : ${channel?.sid}"""
        )
        sendEvent("channelDeleted", mapOf("channel" to SerializationHelper.channelToMap(channel)))
    }

    override fun onInvitedToChannelNotification(channelSID: String?) {
        TwilioChatPlugin.debug("ChatClientListener.onInvitedToChannelNotification => " +
                "channelSID : $channelSID"
        )
        sendEvent("invitedToChannelNotification", mapOf("channelSID" to channelSID))
    }

    override fun onClientSynchronization(syncStatus: ChatClient.SynchronizationStatus?) {
        TwilioChatPlugin.debug("ChatClientListener.onClientSynchronization => " +
                "sync status : $syncStatus"
        )
        sendEvent("clientSynchronization", mapOf("syncStatus" to SerializationHelper.chatClientSyncStatusToMap(syncStatus)))
    }

    override fun onNotificationSubscribed() {
        TwilioChatPlugin.debug("ChatClientListener.onNotificationSubscribed"
        )
        sendEvent("notificationSubscribed", "")
    }

    override fun onUserSubscribed(user: User?) {
        TwilioChatPlugin.debug("ChatClientListener.onUserSubscribed => " +
                "user : ${user?.identity}"
        )
        sendEvent("userSubscribed", mapOf("user" to SerializationHelper.userToMap(user)))
    }

    override fun onChannelUpdated(channel: Channel?, updateReason: Channel.UpdateReason?) {
        TwilioChatPlugin.debug("ChatClientListener.onChannelUpdated => " +
                "channel : ${channel?.sid}," +
                "updateReason : ${updateReason?.name},"
        )
        sendEvent("channelUpdated",
                mapOf("channel" to SerializationHelper.channelToMap(channel),
                        "updateReason" to SerializationHelper.channelUpdateReasonToMap(updateReason)))
    }

    override fun onRemovedFromChannelNotification(channelSID: String?) {
        TwilioChatPlugin.debug("ChatClientListener.onRemovedFromChannelNotification => " +
                "channelSID : $channelSID"
        )
        sendEvent("removedFromChannelNotification", mapOf("channelSID" to channelSID))
    }

    override fun onNotificationFailed(errorInfo: ErrorInfo?) {
        TwilioChatPlugin.debug("ChatClientListener.onNotificationFailed => " +
                "errorCode : ${errorInfo?.code}" +
                "errorMessage : ${errorInfo?.message}"
        )
        sendEvent("notificationFailed", mapOf("error" to SerializationHelper.errorInfoToMap(errorInfo)))
    }

    override fun onTokenExpired() {
        TwilioChatPlugin.debug("ChatClientListener.onTokenExpired"
        )
        sendEvent("tokenExpired", "")
    }

    override fun onChannelJoined(channel: Channel?) {
        TwilioChatPlugin.debug("""ChatClientListener.onChannelJoined => channelSID : ${channel?.sid}"""
        )
        sendEvent("channelJoined", mapOf("channel" to SerializationHelper.channelToMap(channel)))
    }

    override fun onChannelAdded(channel: Channel?) {
        TwilioChatPlugin.debug("""ChatClientListener.onChannelAdded => channelSID : ${channel?.sid}"""
        )
        sendEvent("channelAdded", mapOf("channel" to SerializationHelper.channelToMap(channel)))
    }

    override fun onChannelSynchronizationChange(channel: Channel?) {
        TwilioChatPlugin.debug("""ChatClientListener.onChannelSynchronizationChange => channelSID : ${channel?.sid}"""
        )
        sendEvent("channelSynchronizationChange", mapOf("channel" to SerializationHelper.channelToMap(channel)))
    }

    override fun onUserUnsubscribed(user: User?) {
        TwilioChatPlugin.debug("ChatClientListener.onUserUnsubscribed => " +
                "user : ${user?.identity}"
        )
        sendEvent("userUnsubscribed", mapOf("user" to SerializationHelper.userToMap(user)))
    }

    override fun onAddedToChannelNotification(channelSID: String?) {
        TwilioChatPlugin.debug("ChatClientListener.onAddedToChannelNotification => " +
                "channelSID : $channelSID"
        )
        sendEvent("addedToChannelNotification", mapOf("channelSID" to channelSID))
    }

    override fun onChannelInvited(channel: Channel?) {
        channel?.join(object : StatusListener() {
            override fun onSuccess() {
                TwilioChatPlugin.debug("""ChatClientListener.onChannelInvited => joined : ${channel.sid}""")
            }
        })
        TwilioChatPlugin.debug("""ChatClientListener.onChannelInvited => channelSID : ${channel?.sid}"""
        )
        sendEvent("channelInvited", mapOf("channel" to SerializationHelper.channelToMap(channel)))
    }

    override fun onNewMessageNotification(channelSID: String?, messageSID: String?, messageIndex: Long) {
        TwilioChatPlugin.debug("""ChatClientListener.onNewMessageNotification => 
            |channelSID : $channelSID
            |messageSID : $messageSID
            |messageIndex: $messageIndex
        """.trimMargin()
        )
        sendEvent("newMessageNotification",
                mapOf("channelSID" to channelSID,
                        "messageSID" to messageSID,
                        "messageIndex" to messageIndex))
    }

    override fun onConnectionStateChange(connectionState: ChatClient.ConnectionState?) {
        TwilioChatPlugin.debug("""ChatClientListener.onConnectionStateChange => 
            |connectionState : ${connectionState?.name}
        """.trimMargin()
        )
        sendEvent("connectionStateChange",
                mapOf("connectionState" to SerializationHelper.connectionStateToMap(connectionState)))
    }

    override fun onError(errorInfo: ErrorInfo?) {
        TwilioChatPlugin.debug("ChatClientListener.onError => " +
                "errorCode : ${errorInfo?.code}" +
                "errorMessage : ${errorInfo?.message}"
        )
        sendEvent("error", mapOf("error" to SerializationHelper.errorInfoToMap(errorInfo)))
    }

    override fun onUserUpdated(user: User?, updatedReason: User.UpdateReason?) {
        TwilioChatPlugin.debug("ChatClientListener.onUserUpdated => " +
                "user : ${user?.identity}" +
                "updateReason: ${updatedReason?.name}"
        )

        sendEvent("userUpdated",
                mapOf("user" to SerializationHelper.userToMap(user),
                        "updateReason" to SerializationHelper.userUpdatedReasonToMap(updatedReason)))

    }

    override fun onTokenAboutToExpire() {
        TwilioChatPlugin.debug("ChatClientListener.onTokenAboutToExpire")
        sendEvent("tokenAboutToExpire", "")
    }
}