package tech.krave.twilio_chat_plugin

import com.twilio.chat.*
import java.text.DateFormat
import java.text.SimpleDateFormat

object SerializationHelper {


    fun channelToMap(channel: Channel?): Map<String, Any?> {
        return mapOf(
                "sid" to channel?.sid,
                "type" to channelTypeToMap(channel?.type),
                "createdBy" to channel?.createdBy,
                "dateCreated" to channel?.dateCreated,
                "dateUpdated" to channel?.dateUpdated,
                "friendlyName" to channel?.friendlyName,
                "uniqueName" to channel?.uniqueName,
                "members" to membersToMap(channel?.members)
        )
    }


    fun channelTypeToMap(channelType: Channel.ChannelType?): Map<String, Any?> {
        return mapOf(
                "value" to channelType?.value,
                "name" to channelType?.name
        )
    }


    fun channelDescriptorToMap(channel: ChannelDescriptor?): Map<String, Any?> {
        val df: DateFormat = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSXXX")
        return mapOf(
                "sid" to channel?.sid,
                "createdBy" to channel?.createdBy,
                "dateCreated" to df.format(channel?.dateCreated),
                "dateUpdated" to df.format(channel?.dateUpdated),
                "friendlyName" to channel?.friendlyName,
                "uniqueName" to channel?.uniqueName,
                "membersCount" to channel?.membersCount,
                "messagesCount" to channel?.messagesCount,
                "unconsumedMessagesCount" to channel?.unconsumedMessagesCount
        )
    }


    fun membersToMap(members: Members?): Map<String, Any?> {
        return mapOf(
                "membersList" to members?.membersList?.map { memberToMap(it) }
        )
    }


    fun memberToMap(member: Member?): Map<String, Any?> {
        return mapOf(
                "identity" to member?.identity,
                "sid" to member?.sid,
                "lastConsumptionTimestamp" to member?.lastConsumptionTimestamp,
                "lastConsumedMessageIndex" to member?.lastConsumedMessageIndex
        )
    }


    fun userToMap(user: User?): Map<String, Any?> {
        return mapOf(
                "identity" to user?.identity,
                "friendlyName" to user?.friendlyName,
                "isOnline" to user?.isOnline,
                "isNotifiable" to user?.isNotifiable,
                "isSubscribed" to user?.isSubscribed
        )
    }


    fun chatClientSyncStatusToMap(chatClientSyncStatus: ChatClient.SynchronizationStatus?): Map<String, Any?> {
        return mapOf(
                "value" to chatClientSyncStatus?.value,
                "name" to chatClientSyncStatus?.name
        )
    }


    fun errorInfoToMap(errorInfo: ErrorInfo?): Map<String, Any?> {
        return mapOf(
                "code" to errorInfo?.code,
                "message" to errorInfo?.message,
                "status" to errorInfo?.status
        )
    }


    fun channelUpdateReasonToMap(channelUpdateReason: Channel.UpdateReason?): Map<String, Any?> {
        return mapOf(
                "value" to channelUpdateReason?.value,
                "name" to channelUpdateReason?.name
        )
    }


    fun userUpdatedReasonToMap(userUpdateReason: User.UpdateReason?): Map<String, Any?> {
        return mapOf(
                "value" to userUpdateReason?.value,
                "name" to userUpdateReason?.name
        )
    }


    fun memberUpdatedReasonToMap(memberUpdateReason: Member.UpdateReason?): Map<String, Any?> {
        return mapOf(
                "value" to memberUpdateReason?.value,
                "name" to memberUpdateReason?.name
        )
    }


    fun messageUpdatedReasonToMap(messageUpdateReason: Message.UpdateReason?): Map<String, Any?> {
        return mapOf(
                "value" to messageUpdateReason?.value,
                "name" to messageUpdateReason?.name
        )
    }


    fun connectionStateToMap(connectionState: ChatClient.ConnectionState?): Map<String, Any?> {
        return mapOf(
                "value" to connectionState?.value,
                "name" to connectionState?.name
        )
    }


    fun messageToMap(message: Message?): Map<String, Any?> {
        return mapOf(
                "author" to message?.author,
//                    "channelSID" to message?.channelSid,
                "dateCreated" to message?.dateCreated,
                "memberSID" to message?.memberSid,
                "messageBody" to message?.messageBody,
                "sid" to message?.sid
        )
    }


    fun chatClientConnectionStatusToMap(connectionState: ChatClient.ConnectionState?): Map<String, Any?> {
        return mapOf(
                "name" to connectionState?.name,
                "value" to connectionState?.value
        )
    }


    fun chatClientPropertiesToMap(properties: ChatClient.Properties?): Map<String, Any?> {
        return mapOf(
                "region" to properties?.region,
                "deferCA" to properties?.deferCA
        )
    }


    fun usersToMap(users: Users?): Map<String, Any?> {
        return mapOf(
                "myUser" to userToMap(users?.myUser),
                "subscribedUsers" to users?.subscribedUsers?.map { u -> userToMap(u) }
        )
    }


    fun channelsToMap(channels: Channels?): Map<String, Any?> {
        return mapOf(
                "subscribedChannels" to channels?.subscribedChannels?.map { c -> channelToMap(c) }
        )
    }


    fun chatClientToMap(chatClient: ChatClient?): Map<String, Any?> {
        return mapOf(
                "connectState" to chatClientConnectionStatusToMap(chatClient?.connectionState),
                "isReachabilityEnabled" to chatClient?.isReachabilityEnabled,
                "myIdentity" to chatClient?.myIdentity,
                "users" to usersToMap(chatClient?.users),
                "channels" to channelsToMap(chatClient?.channels),
                "properties" to chatClientPropertiesToMap(chatClient?.properties)
        )
    }
}