import 'package:flutter/material.dart';
import 'package:twilio_chat_plugin_example/shared/helpers/twilio.dart';
import 'package:twilio_chat_plugin/twilio_chat_plugin.dart';

import 'channel.dart';

class ChannelListPage extends StatefulWidget {
  @override
  _ChannelListPageState createState() => _ChannelListPageState();
}

class _ChannelListPageState extends State<ChannelListPage> {
  final chatClient = TwilioHelper.chatClient;
  List<ChannelDescriptor> publicChannelsDescriptors = List<ChannelDescriptor>();
  List<ChannelDescriptor> userChannelsDescriptors = List<ChannelDescriptor>();

  @override
  void initState() {
    super.initState();
    chatClient.listPublicChannels().then((channelDescriptorList) {
      setState(() {
        publicChannelsDescriptors = channelDescriptorList;
      });
    });
    chatClient.listUserChannels().then((channelDescriptorList) {
      setState(() {
        userChannelsDescriptors = channelDescriptorList;
      });
    });
  }

  goIntoChannel(BuildContext context, String channelSID) async {
    final channel = await chatClient.enterChannel(channelSID);
    TwilioHelper.channel = channel;
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (context) => ChannelPage(
                  channel: channel,
                )));
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        leading: SizedBox(height: 0),
        title: const Text('Channel List'),
      ),
      body: ListView(
        children: <Widget>[
          SizedBox(height: 20),
          Container(
              padding: EdgeInsets.all(20),
              child: Text("You\'re signed in as " + chatClient.myIdentity)),
          Container(
            height: 20,
            color: Colors.grey[200],
          ),
          Container(
              padding: EdgeInsets.all(20),
              child: Text(
                'Public Channels',
                style: TextStyle(fontSize: 20, fontWeight: FontWeight.bold),
              )),
          publicChannelsDescriptors.length > 0
              ? ListView.separated(
                  shrinkWrap: true,
                  itemBuilder: (BuildContext context, int index) {
                    return ListTile(
                      title:
                          Text(publicChannelsDescriptors[index].friendlyName),
                      trailing: Text(
                          '${publicChannelsDescriptors[index].unconsumedMessagesCount} unread messages'),
                      onTap: () async {
                        goIntoChannel(
                            context, publicChannelsDescriptors[index].sid);
                      },
                    );
                  },
                  separatorBuilder: (BuildContext context, int index) =>
                      Divider(),
                  itemCount: publicChannelsDescriptors.length)
              : Container(
                  padding: EdgeInsets.all(20),
                  child: Text("No public channels"),
                ),
          Container(
            height: 20,
            color: Colors.grey[200],
          ),
          Container(
              padding: EdgeInsets.all(20),
              child: Text(
                'User Channels',
                style: TextStyle(fontSize: 20, fontWeight: FontWeight.bold),
              )),
          userChannelsDescriptors.length > 0
              ? ListView.separated(
                  shrinkWrap: true,
                  itemBuilder: (BuildContext context, int index) {
                    return ListTile(
                      title: Text(userChannelsDescriptors[index].friendlyName),
                      trailing: Text(
                          '${userChannelsDescriptors[index].unconsumedMessagesCount} unread messages'),
                      onTap: () async {
                        goIntoChannel(
                            context, userChannelsDescriptors[index].sid);
                      },
                    );
                  },
                  separatorBuilder: (BuildContext context, int index) =>
                      Divider(),
                  itemCount: userChannelsDescriptors.length)
              : Container(
                  padding: EdgeInsets.all(20),
                  child: Text("No user channels"),
                ),
        ],
      ),
    );
  }
}
