//
//  ChatClientListener.swift
//  twilio_chat_plugin
//
//  Created by Austin Goh on 21/04/2020.
//

import Foundation
import TwilioChatClient

class ChatClientListener: BaseListener, TwilioChatClientDelegate{
    
    func chatClient(_ client: TwilioChatClient, notificationUpdatedBadgeCount badgeCount: UInt) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.notificationUpdatedBadgeCount => badge count is '\(badgeCount)'")
               sendEvent("channelUserUpdated", data: [ "badgeCount": badgeCount,
                                            ])
    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, updated: TCHChannelUpdate) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.channelUpdated => channel sid is '\(String(describing: channel.sid))'")
        channel.members?.members(){
            res, members in
            var membersListResult = [[String:Any]]()
            for member in (members?.items())! {
                membersListResult.append(SerializationHelper.memberToDict(member))
            }
            self.sendEvent("channelUpdated", data: ["channel": SerializationHelper.channelToDict(channel, membersListResult), "updateReason": SerializationHelper.channelUpdateToDict(updated)])
        }
    }
    
    func chatClient(_ client: TwilioChatClient, connectionStateUpdated state: TCHClientConnectionState) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.connectionStateUpdated => conn state is '\(String(describing: state.rawValue))'")
        sendEvent("connectionStateChange", data: ["connectionState": SerializationHelper.connectionStateToDict(state)])
    }
    
    
    func chatClient(_ client: TwilioChatClient, channelDeleted channel: TCHChannel) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.channelDeleted => channel sid is '\(String(describing: channel.sid))'")
        sendEvent("channelDeleted", data: [ "channel": SerializationHelper.channelToDict(channel, [[String:Any]]()) ])
    }
    
    func chatClient(_ client: TwilioChatClient, notificationInvitedToChannelWithSid channelSid: String) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.notificationInvitedToChannelWithSid => channel sid is '\(String(describing: channelSid))'")
        sendEvent("invitedToChannelNotification", data: [ "channelSID": channelSid ])
        sendEvent("channelInvited", data: [ "channelSID": channelSid ])
    }
    
    func chatClient(_ client: TwilioChatClient, synchronizationStatusUpdated status: TCHClientSynchronizationStatus) {
        if status == TCHClientSynchronizationStatus.completed {
            SwiftTwilioChatPlugin.channelsList = client.channelsList()
            PluginHandler.populateChannels()
        }
        SwiftTwilioChatPlugin.debug("ChatClientListener.synchronizationStatusUpdated => channel sid is '\(status.rawValue)'")
        sendEvent("clientSynchronization", data: [ "syncStatus": SerializationHelper.chatClientSyncStatusToDict(status) ])
    }
    
    func chatClient(_ client: TwilioChatClient, userSubscribed user: TCHUser) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.userSubscribed => user is '\(user.identity ?? "")'")
        sendEvent("userSubscribed", data: [ "user": SerializationHelper.userToDict(user) ])
    }
    
    func chatClient(_ client: TwilioChatClient, notificationRemovedFromChannelWithSid channelSid: String) {
        func notificationRemovedFromChannelWithSid(channelSID: NSString){
            SwiftTwilioChatPlugin.debug("ChatClientListener.removedFromChannelNotification => channel sid is '\(String(describing: channelSID))'")
            sendEvent("removedFromChannelNotification", data: [ "channelSID": channelSID ])
        }
    }
    
    func chatClientTokenExpired(_ client: TwilioChatClient) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.chatClientTokenExpired => token expired")
        sendEvent("tokenExpired", data: [:])
    }
    
    func chatClient(_ client: TwilioChatClient, channelAdded channel: TCHChannel) {
        SwiftTwilioChatPlugin.debug("""
            ChatClientListener.channelAdded => channelSID : '\(String(describing: channel.sid))'
        """
        )
        sendEvent("channelAdded",data: [
            "channel": SerializationHelper.channelToDict(channel, [[String:Any]]())
        ])
    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, synchronizationStatusUpdated status: TCHChannelSynchronizationStatus) {
        SwiftTwilioChatPlugin.debug("""
            ChatClientListener.clientSynchronization => channelSID : '\(String(describing: channel.sid))'${channel?.sid}
        """
        )
        sendEvent("clientSynchronization",data: [
            "channel": SerializationHelper.channelToDict(channel, [[String:Any]]()),
            "status": SerializationHelper.channelSyncStatusToDict(status)
        ])
    }
    
    func chatClient(_ client: TwilioChatClient, userUnsubscribed user: TCHUser) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.userUnsubscribed => user is '\(user.identity ?? "")'")
        sendEvent("userUnsubscribed", data: [ "user": SerializationHelper.userToDict(user) ])
    }
    
    func chatClient(_ client: TwilioChatClient, notificationAddedToChannelWithSid channelSid: String) {
        SwiftTwilioChatPlugin.debug("""
            ChatClientListener.notificationAddedToChannelWithSid => channelSID : '\(String(describing: channelSid))'
        """
        )
        sendEvent("addedToChannelNotification",data: [
            "channelSID": channelSid
        ])
    }
    
    
    func chatClient(_ client: TwilioChatClient, notificationNewMessageReceivedForChannelSid channelSid: String, messageIndex: UInt) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.notificationNewMessageReceivedForChannelSid => channel sid is '\(String(describing: channelSid))'")
        sendEvent("newMessageNotification", data: [ "channelSID": channelSid,
                                                    "messageSID":"",
                                                    "messageIndex": messageIndex
//                                                    "messageSID": messageSid,
        ])
    }
    
    func connectionStateUpdated(state: TCHClientConnectionState) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.connectionStateUpdated => ChatClient connection state is '\(state.rawValue)'")
        sendEvent("connectFailure", data: [ "connectionState": SerializationHelper.connectionStateToDict(state) ])
    }
    
    func chatClient(_ client: TwilioChatClient, errorReceived error: TCHError) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.errorReceived => ChatClient errorCode: '\(error.code)' errorMessage: '\(error.description)'")
        sendEvent("error",data:[
            "error": SerializationHelper.errorToDict(error)
        ])
    }
    
    func chatClient(_ client: TwilioChatClient, user: TCHUser, updated: TCHUserUpdate) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.userUpdated => user is '\(user.identity ?? "")'")
        sendEvent("userUpdated", data: [ "user": SerializationHelper.userToDict(user),
                                         "updateReason": SerializationHelper.userUpdateToDict(updated)])
    }
    
    func chatClientTokenWillExpire(_ client: TwilioChatClient) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.chatClientTokenWillExpire => token will expire")
        sendEvent("tokenAboutToExpire", data: [:])
    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, memberLeft member: TCHMember) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.memberLeft => member is '\(member.identity ?? "")'")
        sendEvent("memberDeleted", data: [
            "member": SerializationHelper.memberToDict(member)
        ])
    }
    
    func chatClient(_ client: TwilioChatClient, typingEndedOn channel: TCHChannel, member: TCHMember) {
        SwiftTwilioChatPlugin.debug("""
            ChatClientListener.typingEndedOn =>
            memberSID : '\(member.sid ?? "")'
            channelSID : '\(channel.sid ?? "")'
        """)
        sendEvent("typingEnded", data: [
            "channel": SerializationHelper.channelToDict(channel, [[String:Any]]()),
            "member": SerializationHelper.memberToDict(member)
        ])
    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, messageAdded message: TCHMessage) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.messageAdded => messageSID : '\(message.sid ?? "")'")
        sendEvent("messageAdded", data: [
            "message": SerializationHelper.messageToDict(message)
        ])
    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, messageDeleted message: TCHMessage) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.messageDeleted => messageSID : '\(message.sid ?? "")'")
        sendEvent("messageDeleted", data: [
            "message": SerializationHelper.messageToDict(message)
        ])
    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, memberJoined member: TCHMember) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.memberJoined => memberSID : '\(member.sid ?? "")'")
        sendEvent("memberAdded", data: [
            "member": SerializationHelper.memberToDict(member)
        ])
    }
    
    func chatClient(_ client: TwilioChatClient, typingStartedOn channel: TCHChannel, member: TCHMember) {
        SwiftTwilioChatPlugin.debug("""
            ChatClientListener.typingStartedOn =>
            memberSID : '\(member.sid ?? "")'
            channelSID : '\(channel.sid ?? "")'
        """)
        sendEvent("typingStarted", data: [
            "channel": SerializationHelper.channelToDict(channel, [[String:Any]]()),
            "member": SerializationHelper.memberToDict(member)
        ])
    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, message: TCHMessage, updated: TCHMessageUpdate) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.messageUpdated => messageSID : '\(message.sid ?? "")'")
        sendEvent("messageUpdated", data: [
            "message": SerializationHelper.messageToDict(message),
            "updateReason": SerializationHelper.messageUpdateToDict(updated)
        ])
    }
    
    func chatClient(_ client: TwilioChatClient, channel: TCHChannel, member: TCHMember, updated: TCHMemberUpdate) {
        SwiftTwilioChatPlugin.debug("ChatClientListener.memberUpdated => memberSID : '\(member.sid ?? "")'")
        sendEvent("memberUpdated", data: [
            "member": SerializationHelper.memberToDict(member),
            "updateReason": SerializationHelper.memberUpdateToDict(updated)
        ])
    }

    
    
}
