part of twilio_chat_plugin;

class Channel {
  String _sid;

  String _createdBy;

  ChannelType _type;

  DateTime _dateCreated;

  DateTime _dateUpdated;

  String _friendlyName;

  String _uniqueName;

  Members _members;

  String get sid => _sid;

  String get createdBy => _createdBy;

  ChannelType get type => _type;

  DateTime get dateCreated => _dateCreated;

  DateTime get dateUpdated => _dateUpdated;

  String get friendlyName => _friendlyName;

  Members get members => _members;

  String get uniqueName => _uniqueName;

  StreamSubscription<dynamic> _channelStream;

  final StreamController<MemberDeletedEvent> _onMemberDeleted =
      StreamController<MemberDeletedEvent>.broadcast();
  Stream<MemberDeletedEvent> onMemberDeleted;

  final StreamController<TypingEndedEvent> _onTypingEnded =
      StreamController<TypingEndedEvent>.broadcast();
  Stream<TypingEndedEvent> onTypingEnded;

  final StreamController<MessageAddedEvent> _onMessageAdded =
      StreamController<MessageAddedEvent>.broadcast();
  Stream<MessageAddedEvent> onMessageAdded;

  final StreamController<MessageDeletedEvent> _onMessageDeleted =
      StreamController<MessageDeletedEvent>.broadcast();
  Stream<MessageDeletedEvent> onMessageDeleted;

  final StreamController<MemberAddedEvent> _onMemberAdded =
      StreamController<MemberAddedEvent>.broadcast();
  Stream<MemberAddedEvent> onMemberAdded;

  final StreamController<TypingStartedEvent> _onTypingStarted =
      StreamController<TypingStartedEvent>.broadcast();
  Stream<TypingStartedEvent> onTypingStarted;

  final StreamController<SynchronizationChangedEvent>
      _onSynchronizationChanged =
      StreamController<SynchronizationChangedEvent>.broadcast();
  Stream<SynchronizationChangedEvent> onSynchronizationChanged;

  final StreamController<MessageUpdatedEvent> _onMessageUpdated =
      StreamController<MessageUpdatedEvent>.broadcast();
  Stream<MessageUpdatedEvent> onMessageUpdated;

  final StreamController<MemberUpdatedEvent> _onMemberUpdated =
      StreamController<MemberUpdatedEvent>.broadcast();
  Stream<MemberUpdatedEvent> onMemberUpdated;

  final StreamController<ErrorEvent> _onError =
      StreamController<ErrorEvent>.broadcast();
  Stream<ErrorEvent> onError;

  Channel.fromJson(Map<String, dynamic> json) {
    _sid = json['sid'];
    _friendlyName = json['friendlyName'];
    _uniqueName = json['uniqueName'];
    _type = ChannelType.fromJson(Map<String, dynamic>.from(json['type']));
    if (json['dateCreated'] != null && json['dateCreated'] != "") {
      _dateCreated = datetimeFormat.parse(json['dateCreated']);
    } else {
      _dateCreated = null;
    }
    if (json['dateUpdated'] != null && json['dateUpdated'] != "") {
      _dateUpdated = datetimeFormat.parse(json['dateUpdated']);
    } else {
      _dateUpdated = null;
    }

    _createdBy = json['createdBy'];
    _members = json['members'] != null
        ? new Members.fromJson(Map<String, dynamic>.from(json['members']))
        : null;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['sid'] = this.sid;
    data['friendlyName'] = this.friendlyName;
    data['uniqueName'] = this.uniqueName;
    data['type'] = this.type.toJson();
    data['dateCreated'] = datetimeFormat.format(this.dateCreated);
    data['dateUpdated'] = datetimeFormat.format(this.dateUpdated);
    data['createdBy'] = this.createdBy;
    data['members'] = this.members.toJson();
    return data;
  }

  void attachListener() {
    if (_channelStream == null) {
      _channelStream = TwilioChatPlugin._channelChannel
          .receiveBroadcastStream()
          .listen(_parseChannelEvents);
    }
    onMemberDeleted = _onMemberDeleted.stream;
    onTypingEnded = _onTypingEnded.stream;
    onMessageAdded = _onMessageAdded.stream;
    onMessageDeleted = _onMessageDeleted.stream;
    onMemberAdded = _onMemberAdded.stream;
    onTypingStarted = _onTypingStarted.stream;
    onSynchronizationChanged = _onSynchronizationChanged.stream;
    onMessageUpdated = _onMessageUpdated.stream;
    onMemberUpdated = _onMemberUpdated.stream;
  }

  void detachListener() async {
    _channelStream.cancel();
    _channelStream = null;
    await this.exitChannel();
  }

  void _parseChannelEvents(dynamic event) {
    final String eventName = event['name'];
    final data = Map<String, dynamic>.from(event['data']);

    if (event['error'] != null) {
      final errorMap =
          Map<String, dynamic>.from(event['error'] as Map<dynamic, dynamic>);
      _onError.add(ErrorEvent(ErrorInfo.fromJson(errorMap)));
      return;
    }
    switch (eventName) {
      case 'memberDeleted':
        if (data['member'] == null) return;
        final memberMap = Map<String, dynamic>.from(data['member']);
        final member = Member.fromJson(memberMap);
        _onMemberDeleted.add(MemberDeletedEvent(member));
        break;
      case 'typingEnded':
        if (data['channel'] == null || data['member'] == null) return;
        final channelMap = Map<String, dynamic>.from(data['channel']);
        final channel = Channel.fromJson(channelMap);
        final memberMap = Map<String, dynamic>.from(data['member']);
        final member = Member.fromJson(memberMap);
        _onTypingEnded.add(TypingEndedEvent(channel, member));
        break;
      case 'messageAdded':
        if (data['message'] == null) return;
        final messageMap = Map<String, dynamic>.from(data['message']);
        final message = Message.fromJson(messageMap);
        _onMessageAdded.add(MessageAddedEvent(message));
        break;
      case 'messageDeleted':
        if (data['message'] == null) return;
        final messageMap = Map<String, dynamic>.from(data['message']);
        final message = Message.fromJson(messageMap);
        _onMessageDeleted.add(MessageDeletedEvent(message));
        break;
      case 'memberAdded':
        if (data['member'] == null) return;
        final memberMap = Map<String, dynamic>.from(data['member']);
        final member = Member.fromJson(memberMap);
        _onMemberAdded.add(MemberAddedEvent(member));
        break;
      case 'typingStarted':
        if (data['channel'] == null || data['member'] == null) return;
        final channelMap = Map<String, dynamic>.from(data['channel']);
        final channel = Channel.fromJson(channelMap);
        final memberMap = Map<String, dynamic>.from(data['member']);
        final member = Member.fromJson(memberMap);
        _onTypingStarted.add(TypingStartedEvent(channel, member));
        break;
//      case 'channelSynchronizationChanged':
//        if (data['channel'] == null) return;
//        final channelMap = Map<String, dynamic>.from(data['channel']);
//        final channel = Channel.fromJson(channelMap);
//        _onChannelSynchronizationChange
//            .add(ChannelSynchronizationChangedEvent(channel));
//        break;
      case 'messageUpdated':
        if (data['message'] == null || data['updateReason'] == null) return;
        final messageMap = Map<String, dynamic>.from(data['message']);
        final message = Message.fromJson(messageMap);
        final updateReasonMap = Map<String, dynamic>.from(data['updateReason']);
        final updateReason = MessageUpdateReason.fromJson(updateReasonMap);
        _onMessageUpdated.add(MessageUpdatedEvent(message, updateReason));
        break;
      case 'memberUpdated':
        if (data['member'] == null || data['updateReason'] == null) return;
        final memberMap = Map<String, dynamic>.from(data['member']);
        final member = Member.fromJson(memberMap);
        final updateReasonMap = Map<String, dynamic>.from(data['updateReason']);
        final updateReason = MemberUpdateReason.fromJson(updateReasonMap);
        _onMemberUpdated.add(MemberUpdatedEvent(member, updateReason));
        break;
    }
  }

  sendMessagesByChannelSID(String messageBody) async {
    return await TwilioChatPlugin._methodChannel.invokeMethod(
        'sendMessagesByChannelSID',
        {'channelSID': this._sid, 'messageBody': messageBody});
  }

  Future<List<Message>> getMostRecentMessagesByChannelSID(
      int messageCount) async {
    final res = await TwilioChatPlugin._methodChannel.invokeMethod(
        'getMostRecentMessagesByChannelSID',
        {'channelSID': this._sid, 'messageCount': messageCount});
    final messagesMap =
        List.from(res).map((r) => Map<String, dynamic>.from(r)).toList();
    return messagesMap.map((c) => Message.fromJson(c)).toList();
  }

  Future<List<Message>> getMessagesBefore(int messageCount, int before) async {
    final res = await TwilioChatPlugin._methodChannel.invokeMethod(
        'getMessagesBefore', {
      'channelSID': this._sid,
      'messageCount': messageCount,
      'before': before
    });
    final messagesMap =
        List.from(res).map((r) => Map<String, dynamic>.from(r)).toList();
    return messagesMap.map((c) => Message.fromJson(c)).toList();
  }

  Future<Channel> joinChannel() async {
    final res = await TwilioChatPlugin._methodChannel
        .invokeMethod('joinChannel', {'channelSID': this._sid});
    final channelMap = Map<String, dynamic>.from(res);
    return Channel.fromJson(channelMap);
  }

  Future<bool> inviteUserToChannel(String invited) async {
    final res = await TwilioChatPlugin._methodChannel
        .invokeMethod('inviteUserToChannel',
            {'channelSID': this._sid, 'identity': invited})
        .timeout(Duration(seconds: 10))
        .catchError(() {
          return false;
        });
    return res;
  }

  Future<bool> updateChannelFriendlyName(String channelName) async {
    final res = await TwilioChatPlugin._methodChannel
        .invokeMethod('updateChannelFriendlyName', {
          'friendlyName': channelName,
          'channelSID': this._sid,
        })
        .timeout(Duration(seconds: 10))
        .catchError(() {
          return false;
        });
    return res;
  }

  leaveChannel() async {
    return await TwilioChatPlugin._methodChannel
        .invokeMethod('leaveChannel', {'channelSID': this._sid});
  }

  Future<bool> exitChannel() async {
    final res =
        await TwilioChatPlugin._methodChannel.invokeMethod('exitChannel');
    return res;
  }
}
