part of twilio_chat_plugin;

class ChatClient {
  StreamSubscription<dynamic> _chatClientStream;

  ChatClientConnectionState _connectState;

  bool _isReachabilityEnabled;

  String _myIdentity;

  Users _users;

  Channels _channels;

  ChatClientProperties _chatClientProperties;

  String _fcmToken;

  StreamSubscription<dynamic> get chatClientStream => _chatClientStream;

  ChatClientConnectionState get connectState => _connectState;

  ChatClientProperties get chatClientProperties => _chatClientProperties;

  Channels get channels => _channels;

  Users get users => _users;

  String get myIdentity => _myIdentity;

  String get fcmToken => _fcmToken;

  bool get isReachabilityEnabled => _isReachabilityEnabled;

  final StreamController<ChannelDeletedEvent> _onChannelDeleted =
      StreamController<ChannelDeletedEvent>.broadcast();
  Stream<ChannelDeletedEvent> onChannelDeleted;

  final StreamController<InvitedToChannelNotificationEvent>
      _onInvitedToChannelNotification =
      StreamController<InvitedToChannelNotificationEvent>.broadcast();
  Stream<InvitedToChannelNotificationEvent> onInvitedToChannelNotification;

  final StreamController<ClientSynchronizationEvent> _onClientSynchronization =
      StreamController<ClientSynchronizationEvent>.broadcast();
  Stream<ClientSynchronizationEvent> onClientSynchronization;

  final StreamController<NotificationSubscribedEvent>
      _onNotificationSubscribed =
      StreamController<NotificationSubscribedEvent>.broadcast();
  Stream<NotificationSubscribedEvent> onNotificationSubscribed;

  final StreamController<UserSubscribedEvent> _onUserSubscribed =
      StreamController<UserSubscribedEvent>.broadcast();
  Stream<UserSubscribedEvent> onUserSubscribed;

  final StreamController<ChannelUpdatedEvent> _onChannelUpdated =
      StreamController<ChannelUpdatedEvent>.broadcast();
  Stream<ChannelUpdatedEvent> onChannelUpdated;

  final StreamController<RemovedFromChannelNotificationEvent>
      _onRemovedFromChannelNotification =
      StreamController<RemovedFromChannelNotificationEvent>.broadcast();
  Stream<RemovedFromChannelNotificationEvent> onRemovedFromChannelNotification;

  final StreamController<NotificationFailedEvent> _onNotificationFailed =
      StreamController<NotificationFailedEvent>.broadcast();
  Stream<NotificationFailedEvent> onNotificationFailed;

  final StreamController<TokenExpiredEvent> _onTokenExpired =
      StreamController<TokenExpiredEvent>.broadcast();
  Stream<TokenExpiredEvent> onTokenExpired;

  final StreamController<ChannelJoinedEvent> _onChannelJoined =
      StreamController<ChannelJoinedEvent>.broadcast();
  Stream<ChannelJoinedEvent> onChannelJoined;

  final StreamController<ChannelAddedEvent> _onChannelAdded =
      StreamController<ChannelAddedEvent>.broadcast();
  Stream<ChannelAddedEvent> onChannelAdded;

  final StreamController<ChannelSynchronizationChangedEvent>
      _onChannelSynchronizationChange =
      StreamController<ChannelSynchronizationChangedEvent>.broadcast();
  Stream<ChannelSynchronizationChangedEvent> onChannelSynchronizationChange;

  final StreamController<UserUnsubscribedEvent> _onUserUnsubscribed =
      StreamController<UserUnsubscribedEvent>.broadcast();
  Stream<UserUnsubscribedEvent> onUserUnsubscribed;

  final StreamController<AddedToChannelNotificationEvent>
      _onAddedToChannelNotification =
      StreamController<AddedToChannelNotificationEvent>.broadcast();
  Stream<AddedToChannelNotificationEvent> onAddedToChannelNotification;

  final StreamController<ChannelInvitedEvent> _onChannelInvited =
      StreamController<ChannelInvitedEvent>.broadcast();
  Stream<ChannelInvitedEvent> onChannelInvited;

  final StreamController<NewMessageNotificationEvent>
      _onNewMessageNotification =
      StreamController<NewMessageNotificationEvent>.broadcast();
  Stream<NewMessageNotificationEvent> onNewMessageNotification;

  final StreamController<ConnectionStateChangedEvent> _onConnectionStateChange =
      StreamController<ConnectionStateChangedEvent>.broadcast();
  Stream<ConnectionStateChangedEvent> onConnectionStateChange;

  final StreamController<ErrorEvent> _onError =
      StreamController<ErrorEvent>.broadcast();
  Stream<ErrorEvent> onError;

  final StreamController<UserUpdatedEvent> _onUserUpdated =
      StreamController<UserUpdatedEvent>.broadcast();
  Stream<UserUpdatedEvent> onUserUpdated;

  final StreamController<TokenAboutToExpireEvent> _onTokenAboutToExpire =
      StreamController<TokenAboutToExpireEvent>.broadcast();
  Stream<TokenAboutToExpireEvent> onTokenAboutToExpire;

  void attachListener() {
    if (_chatClientStream == null) {
      _chatClientStream = TwilioChatPlugin._chatClientChannel
          .receiveBroadcastStream()
          .listen(_parseChatClientEvents);
    }

    onChannelDeleted = _onChannelDeleted.stream;
    onInvitedToChannelNotification = _onInvitedToChannelNotification.stream;
    onClientSynchronization = _onClientSynchronization.stream;
    onNotificationSubscribed = _onNotificationSubscribed.stream;
    onUserSubscribed = _onUserSubscribed.stream;
    onChannelUpdated = _onChannelUpdated.stream;
    onRemovedFromChannelNotification = _onRemovedFromChannelNotification.stream;
    onNotificationFailed = _onNotificationFailed.stream;
    onTokenExpired = _onTokenExpired.stream;
    onChannelJoined = _onChannelJoined.stream;
    onChannelAdded = _onChannelAdded.stream;
    onChannelSynchronizationChange = _onChannelSynchronizationChange.stream;
    onUserUnsubscribed = _onUserUnsubscribed.stream;
    onAddedToChannelNotification = _onAddedToChannelNotification.stream;
    onChannelInvited = _onChannelInvited.stream;
    onNewMessageNotification = _onNewMessageNotification.stream;
    onConnectionStateChange = _onConnectionStateChange.stream;
    onError = _onError.stream;
    onUserUpdated = _onUserUpdated.stream;
    onTokenAboutToExpire = _onTokenAboutToExpire.stream;
  }

  void detachListener() {
    if (_chatClientStream != null) {
      _chatClientStream.cancel();
      _chatClientStream = null;
    }
  }

  void _parseChatClientEvents(dynamic event) {
    final String eventName = event['name'];
    TwilioChatPlugin._log(
        "ChatClient => Event '$eventName' => ${event["data"]}, error: ${event["error"]}");
    var data;
    try {
      data = Map<String, dynamic>.from(event['data']);
    } catch (e) {
      data = event['data'];
    }

    if (event['error'] != null) {
      final errorMap =
          Map<String, dynamic>.from(event['error'] as Map<dynamic, dynamic>);
      _onError.add(ErrorEvent(ErrorInfo.fromJson(errorMap)));
      return;
    }
    switch (eventName) {
      case 'channelSynchronizationChanged':
        if (data['channel'] == null) return;
        final channelMap = Map<String, dynamic>.from(data['channel']);
        final channel = Channel.fromJson(channelMap);
        _onChannelSynchronizationChange
            .add(ChannelSynchronizationChangedEvent(channel));
        break;
      case 'channelDeleted':
        if (data['channel'] == null) return;
        final channelMap = Map<String, dynamic>.from(data['channel']);
        final channel = Channel.fromJson(channelMap);
        _onChannelDeleted.add(ChannelDeletedEvent(channel));
        break;
      case 'invitedToChannelNotification':
        if (data['channelSID'] == null) return;
        final channelSID = data['channelSID'];
        _onInvitedToChannelNotification
            .add(InvitedToChannelNotificationEvent(channelSID));
        break;
      case 'clientSynchronization':
        if (data['syncStatus'] == null) return;
        final syncStatusMap = Map<String, dynamic>.from(data['syncStatus']);
        final syncStatus =
            ChatClientSynchronizationStatus.fromJson(syncStatusMap);
        _onClientSynchronization.add(ClientSynchronizationEvent(syncStatus));
        break;
      case 'notificationSubscribed':
        _onNotificationSubscribed.add(NotificationSubscribedEvent());
        break;
      case 'userSubscribed':
        if (data['user'] == null) return;
        final userMap = Map<String, dynamic>.from(data['user']);
        final user = User.fromJson(userMap);
        _onUserSubscribed.add(UserSubscribedEvent(user));
        break;
      case 'channelUpdated':
        if (data['channel'] == null || data['updateReason'] != null) return;
        final channelMap = Map<String, dynamic>.from(data['channel']);
        final updateReasonMap = Map<String, dynamic>.from(data['updateReason']);
        final channel = Channel.fromJson(channelMap);
        final updateReason = ChannelUpdateReason.fromJson(updateReasonMap);
        _onChannelUpdated.add(ChannelUpdatedEvent(channel, updateReason));
        break;
      case 'removedFromChannelNotification':
        if (data['channelSID'] == null) return;
        final channelSID = data['channelSID'];
        _onRemovedFromChannelNotification
            .add(RemovedFromChannelNotificationEvent(channelSID));
        break;
      case 'notificationFailed':
        if (data['error'] == null) return;
        final errorMap = Map<String, dynamic>.from(data['error']);
        final error = ErrorInfo.fromJson(errorMap);
        _onNotificationFailed.add(NotificationFailedEvent(error));
        break;
      case 'tokenExpired':
        _onTokenExpired.add(TokenExpiredEvent());
        break;
      case 'channelJoined':
        if (data['channel'] == null) return;
        final channelMap = Map<String, dynamic>.from(data['channel']);
        final channel = Channel.fromJson(channelMap);
        _onChannelJoined.add(ChannelJoinedEvent(channel));
        break;
      case 'channelAdded':
        if (data['channel'] == null) return;
        final channelMap = Map<String, dynamic>.from(data['channel']);
        final channel = Channel.fromJson(channelMap);
        _onChannelAdded.add(ChannelAddedEvent(channel));
        break;
      case 'channelSynchronizationChange':
        if (data['channel'] == null) return;
        final channelMap = Map<String, dynamic>.from(data['channel']);
        final channel = Channel.fromJson(channelMap);
        _onChannelSynchronizationChange
            .add(ChannelSynchronizationChangedEvent(channel));
        break;
      case 'userUnsubscribed':
        if (data['user'] == null) return;
        final userMap = Map<String, dynamic>.from(data['user']);
        final user = User.fromJson(userMap);
        _onUserUnsubscribed.add(UserUnsubscribedEvent(user));
        break;
      case 'addedToChannelNotification':
        if (data['channelSID'] == null) return;
        final channelSID = data['channelSID'];
        _onAddedToChannelNotification
            .add(AddedToChannelNotificationEvent(channelSID));
        break;
      case 'channelInvited':
        if (data['channel'] == null) return;
        final channelMap = Map<String, dynamic>.from(data['channel']);
        final channel = Channel.fromJson(channelMap);
        _onChannelInvited.add(ChannelInvitedEvent(channel));
        break;
      case 'newMessageNotification':
        if (data['channelSID'] == null ||
            data['messageSID'] == null ||
            data['messageIndex'] == null) return;
        final channelSID = data['channelSID'];
        final messageSID = data['channelSID'];
        final messageIndex = int.tryParse(data['channelSID'], radix: 10);
        _onNewMessageNotification.add(
            NewMessageNotificationEvent(channelSID, messageSID, messageIndex));
        break;
      case 'connectionStateChange':
        if (data['connectionState'] == null) return;
        final connectionStateMap =
            Map<String, dynamic>.from(data['connectionState']);
        final connectionState =
            ChatClientConnectionState.fromJson(connectionStateMap);
        _onConnectionStateChange
            .add(ConnectionStateChangedEvent(connectionState));
        break;
      case 'error':
        if (data['error'] == null) return;
        final errorMap = Map<String, dynamic>.from(data['error']);
        final error = ErrorInfo.fromJson(errorMap);
        _onError.add(ErrorEvent(error));
        break;
      case 'userUpdated':
        if (data['user'] == null || data['updateReason'] == null) return;
        final userMap = Map<String, dynamic>.from(data['user']);
        final user = User.fromJson(userMap);
        final updateReasonMap = Map<String, dynamic>.from(data['updateReason']);
        final updateReason = UserUpdateReason.fromJson(updateReasonMap);
        _onUserUpdated.add(UserUpdatedEvent(user, updateReason));
        break;
      case 'tokenAboutToExpire':
        _onTokenAboutToExpire.add(TokenAboutToExpireEvent());
        break;
    }
  }

  registerFCMToken(String token) async {
    _fcmToken = token;
    return await TwilioChatPlugin._methodChannel
        .invokeMethod('registerFCMToken', {'fcmToken': token});
  }

  unregisterFCMToken(String token) async {
    _fcmToken = null;
    return await TwilioChatPlugin._methodChannel
        .invokeMethod('unregisterFCMToken', {'fcmToken': token});
  }

  replaceFCMToken(String oldFcmToken, String newFcmToken) async {
    _fcmToken = newFcmToken;
    return await TwilioChatPlugin._methodChannel.invokeMethod('replaceFCMToken',
        {'oldFcmToken': oldFcmToken, 'newFcmToken': newFcmToken});
  }

  Future<Channel> enterChannel(String channelSID) async {
    final res = await TwilioChatPlugin._methodChannel
        .invokeMethod('enterChannel', {'channelSID': channelSID});
    final channelMap = Map<String, dynamic>.from(res);
    final channel = Channel.fromJson(channelMap);
    return channel;
  }

  Future<bool> exitChannel(String channelSID) async {
    final res = await TwilioChatPlugin._methodChannel
        .invokeMethod('exitChannel', {'channelSID': channelSID});
    return res;
  }

  Future<List<ChannelDescriptor>> listPublicChannels() async {
    final res = await TwilioChatPlugin._methodChannel
        .invokeMethod('listPublicChannels');
    final publicChannelsMap =
        List.from(res).map((r) => Map<String, dynamic>.from(r)).toList();
    return publicChannelsMap.map((c) => ChannelDescriptor.fromJson(c)).toList();
  }

  Future<List<ChannelDescriptor>> listUserChannels() async {
    final res =
        await TwilioChatPlugin._methodChannel.invokeMethod('listUserChannels');
    final userChannelsMap =
        List.from(res).map((r) => Map<String, dynamic>.from(r)).toList();
    return userChannelsMap.map((c) => ChannelDescriptor.fromJson(c)).toList();
  }

  Future<bool> updateUserFriendlyName(String friendlyName) async {
    final res = await TwilioChatPlugin._methodChannel
        .invokeMethod('updateUserFriendlyName', {'friendlyName': friendlyName})
        .timeout(Duration(seconds: 10))
        .catchError(() {
          return false;
        });
    return res;
  }

  Future<Channel> createPrivateChannelByIdentites(
      List<String> userIdentities) async {
    final res = await TwilioChatPlugin._methodChannel.invokeMethod(
        'createPrivateChannelByIdentites', {'userIdentities': userIdentities});
    final channelMap = Map<String, dynamic>.from(res);
    return Channel.fromJson(channelMap);
  }

  ChatClient.fromJson(Map<String, dynamic> json) {
    _connectState = json['connectState'] != null
        ? new ChatClientConnectionState.fromJson(
            Map<String, dynamic>.from(json['connectState']))
        : null;

    _isReachabilityEnabled = json['isReachabilityEnabled'];
    _myIdentity = json['myIdentity'];
    _users = json['users'] != null
        ? new Users.fromJson(Map<String, dynamic>.from(json['users']))
        : null;
    _channels = json['channels'] != null
        ? new Channels.fromJson(Map<String, dynamic>.from(json['channels']))
        : null;
    _chatClientProperties = json['properties'] != null
        ? new ChatClientProperties.fromJson(
            Map<String, dynamic>.from(json['properties']))
        : null;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    if (this.connectState != null) {
      data['connectState'] = this.connectState.toJson();
    }
    data['isReachabilityEnabled'] = this.isReachabilityEnabled;
    data['myIdentity'] = this.myIdentity;
    if (this.users != null) {
      data['users'] = this.users.toJson();
    }
    if (this.channels != null) {
      data['channels'] = this.channels.toJson();
    }
    if (this.chatClientProperties != null) {
      data['properties'] = this._chatClientProperties.toJson();
    }
    return data;
  }
}
