part of twilio_chat_plugin;

class TwilioChatPlugin {
  static const MethodChannel _methodChannel =
      MethodChannel('krave.tech/twilio_chat_plugin');

  static const EventChannel _chatClientChannel =
      EventChannel('krave.tech/twilio_chat_plugin/chat_client');

  static const EventChannel _channelChannel =
      EventChannel('krave.tech/twilio_chat_plugin/channel');

  static const EventChannel _loggingChannel =
      EventChannel('krave.tech/twilio_chat_plugin/logging');

  static StreamSubscription _loggingStream;

  static var _dartDebug = true;

  static void _log(dynamic msg) {
    if (_dartDebug) {
      print('[   DART   ] $msg');
    }
  }

  static Future<void> debug({bool dart = true, bool native = true}) async {
    assert(dart != null);
    assert(native != null);
    _dartDebug = dart;
    await _methodChannel.invokeMethod('debug', {'native': native});
    if (native && _loggingStream == null) {
      _loggingStream =
          _loggingChannel.receiveBroadcastStream().listen((dynamic event) {
        if (native) {
          print('[  NATIVE  ] $event');
        }
      });
    } else if (!native && _loggingStream != null) {
      await _loggingStream.cancel();
      _loggingStream = null;
    }
  }

  static Future<ChatClient> createChatClient(
      String accessToken, String fcmToken) async {
    final result = await _methodChannel
        .invokeMethod('createChatClient', {'accessToken': accessToken});
    final chatClientMap = Map<String, dynamic>.from(result);
    final chatClient = ChatClient.fromJson(chatClientMap);
    return chatClient;
  }

  static createChannel(String friendlyName, int channelType) async {
    return await _methodChannel.invokeMethod('createChannel',
        {'friendlyName': friendlyName, 'channelType': channelType});
  }
}
